/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qregexp.h>
#include <qstringlist.h>

#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <encoderxml.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include "bibtexfileexporterxml.h"

namespace BibTeX
{

    BibTeXFileExporterXML::BibTeXFileExporterXML()
            : BibTeXFileExporter()
    {
        cancelFlag = FALSE;
    }


    BibTeXFileExporterXML::~BibTeXFileExporterXML()
    {
        // nothing
    }

    bool BibTeXFileExporterXML::save( QIODevice* iodevice, BibTeXFile* bibtexfile )
    {
        bool result = TRUE;
        QTextStream stream( iodevice );

        stream << "<?xml version=\"1.0\"?>" << endl;
        stream << "<bibliography>" << endl;

        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXElement *element = *it;
            write( stream, element );
        }

        stream << "</bibliography>" << endl;

        return result && !cancelFlag;
    }

    bool BibTeXFileExporterXML::save( QIODevice* iodevice, BibTeXElement* element )
    {
        QTextStream stream( iodevice );

        stream << "<?xml version=\"1.0\"?>" << endl;
        return write( stream, element );
    }

    void BibTeXFileExporterXML::cancel()
    {
        cancelFlag = TRUE;
    }

    bool BibTeXFileExporterXML::write( QTextStream&stream, BibTeXElement *element )
    {
        bool result = FALSE;

        BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( element );
        if ( entry != NULL )
            result |= writeEntry( stream, entry );
        else
        {
            BibTeXString * string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                result |= writeString( stream, string );
            else
            {
                BibTeXComment * comment = dynamic_cast<BibTeXComment*>( element );
                if ( comment != NULL )
                    result |= writeComment( stream, comment );
            }
        }

        return result;
    }

    bool BibTeXFileExporterXML::writeEntry( QTextStream &stream, BibTeXEntry* entry )
    {
        stream << " <entry id=\"" << entry->id() << "\" type=\"" << entry->entryTypeString().lower() << "\">" << endl;
        for ( BibTeXEntry::EntryFields::iterator it = entry->begin(); it != entry->end(); it++ )
        {
            BibTeXEntryField *field = *it;
            switch ( field->fieldType() )
            {
            case BibTeXEntryField::ftAuthor:
            case BibTeXEntryField::ftEditor:
                {
                    QString tag = field->fieldTypeName().lower();
                    stream << "  <" << tag << "s>" << endl;
                    QStringList persons = QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", FALSE ), EncoderXML::currentEncoderXML() ->encode( itemToString( field->begin(), field->end() ) ) );
                    for ( QStringList::Iterator it = persons.begin(); it != persons.end(); it++ )
                        stream << "   <person>" << *it << "</person>" << endl;
                    stream << "  </" << tag << "s>" << endl;
                }
                break;
            default:
                {
                    QString tag = field->fieldTypeName().lower();
                    stream << "  <" << tag << ">" << EncoderXML::currentEncoderXML() ->encode( itemToString( field->begin(), field->end() ) ) << "</" << tag << ">" << endl;
                }
                break;
            }

        }
        stream << " </entry>" << endl;

        return TRUE;
    }

    bool BibTeXFileExporterXML::writeString( QTextStream &stream, BibTeXString *string )
    {
        stream << " <string key=\"" << string->key() << "\">";
        stream << EncoderXML::currentEncoderXML() ->encode( itemToString( string->begin(), string->end() ) );
        stream << "</string>" << endl;

        return TRUE;
    }

    bool BibTeXFileExporterXML::writeComment( QTextStream &stream, BibTeXComment *comment )
    {
        stream << " <comment>" ;
        stream << EncoderXML::currentEncoderXML() ->encode( comment->text() );
        stream << "</comment>" << endl;

        return TRUE;
    }

    QString BibTeXFileExporterXML::itemToString( const BibTeXElement::FieldItems::iterator& begin, const BibTeXElement::FieldItems::iterator& end )
    {
        QString result;
        bool isFirst = TRUE;

        for ( BibTeXElement::FieldItems::const_iterator it = begin; it != end; it++ )
        {
            if ( !isFirst )
                result.append( ' ' );

            result.append( ( *it ).text );

            isFirst = FALSE;
        }

        return result;
    }


}
