// $id$

/***************************************************************************
 *   Copyright (C) 2004-2005 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <bibtexxsltransform.h>

#include "bibtexstring.h"

#define max(a,b) ((a)>(b)?(a):(b))

namespace BibTeX
{

    BibTeXString::BibTeXString( const QString &key )
            : BibTeXElement(), m_key( key )
    {
        // nothing
    }

    BibTeXString::BibTeXString( BibTeXString *other )
    {
        m_key = other->m_key;
        for ( BibTeXElement::FieldItems::iterator it = other->m_stringItems.begin();it != other->m_stringItems.end(); it++ )
        {
            BibTeXElement::FieldItem item;
            item.text = ( *it ).text;
            item.isStringKey = ( *it ).isStringKey;
            m_stringItems.append( item );
        }
    }

    BibTeXString::~BibTeXString()
    {
        // nothing
    }

    void BibTeXString::setKey( const QString &key )
    {
        m_key = key;
    }
    QString BibTeXString::key()

    {
        return m_key;
    }

    void BibTeXString::appendStringItem( const BibTeXElement::FieldItem& stringItem )
    {
        m_stringItems.append( stringItem );
    }

    void BibTeXString::clearStringItems()
    {
        m_stringItems.clear();
    }

    BibTeXElement::FieldItems::iterator BibTeXString::begin()
    {
        return m_stringItems.begin();
    }

    BibTeXElement::FieldItems::iterator BibTeXString::end()
    {
        return m_stringItems.end();
    }

    bool BibTeXString::containsPattern( const QString& pattern, bool caseSensitive )
    {
        bool result = m_key.find( pattern, 0, caseSensitive ) > 0;

        for ( BibTeXElement::FieldItems::iterator it = m_stringItems.begin(); !result && it != m_stringItems.end(); it++ )
            result |= ( *it ).text.find( pattern, 0, caseSensitive ) > 0;

        return result;
    }

}
