/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qregexp.h>

#include "encoderxml.h"

namespace BibTeX
{
    EncoderXML *encoderXML = NULL;

    static const struct EncoderXMLCharMapping
    {
        const char *regexp;
        unsigned int unicode;
        const char *latex;
    }
    charmappingdata[] = {
                            {"&amp;", 0x0026, "&amp;"},
                        };
    static const int charmappingdatacount = sizeof( charmappingdata ) / sizeof( charmappingdata[ 0 ] ) ;

    EncoderXML::EncoderXML()
            : Encoder()
    {
        buildCharMapping();
    }


    EncoderXML::~EncoderXML()
    {
        freeCharMapping();
    }

    QString EncoderXML::decode( const QString &text )
    {
        QString result = text;

        for ( CharMappingItem * charMappingItem = charMapping->first(); charMappingItem; charMappingItem = charMapping->next() )
            result.replace( *( charMappingItem->regExp ), *( charMappingItem->unicode ) );

        return result;
    }

    QString EncoderXML::encode( const QString &text )
    {
        QString result = text;

        for ( CharMappingItem * charMappingItem = charMapping->first(); charMappingItem; charMappingItem = charMapping->next() )
            result.replace( *( charMappingItem->unicode ), *( charMappingItem->latex ) );

        return result;
    }

    QString EncoderXML::decodeSpecialized( const QString &text, const BibTeXEntryField::FieldType /* fieldType */ )
    {
        return decode( text );
    }

    QString EncoderXML::encodeSpecialized( const QString &text, const BibTeXEntryField::FieldType  /* fieldType */ )
    {
        return encode( text );
    }

    void EncoderXML::buildCharMapping()
    {
        charMapping = new QPtrList<CharMappingItem>();

        for ( int i = 0; i < charmappingdatacount; i++ )
        {
            CharMappingItem * charMappingItem = new CharMappingItem;
            charMappingItem->regExp = new QRegExp( charmappingdata[ i ].regexp );
            charMappingItem->unicode = new QChar( charmappingdata[ i ].unicode );
            charMappingItem->latex = new QString( charmappingdata[ i ].latex );
            charMapping->append( charMappingItem );
        }
    }

    void EncoderXML::freeCharMapping()
    {
        CharMappingItem * charMappingItem = NULL;

        while ( ( charMappingItem = charMapping->first() ) != NULL )
        {
            delete charMappingItem->regExp;
            delete charMappingItem->unicode;
            delete charMappingItem->latex;
            charMapping->remove();
            delete charMappingItem;
        }

        delete charMapping;
        charMapping = NULL;
    }

    EncoderXML *EncoderXML::currentEncoderXML()
    {
        if ( encoderXML == NULL )
            encoderXML = new EncoderXML();

        return encoderXML;
    }

}
