/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KBIBTEXLISTVIEW_H
#define KBIBTEXLISTVIEW_H

#include <klistview.h>

#include <bibtexfileimporter.h>
#include <settings.h>
#include <bibtexfile.h>

class QString;
class QProgressDialog;
class KURL;
class KBibTeXPart;
class KBibTeXListViewItem;
// class BibTeXFileImporter;

static const QString Columns[] =
    {
        QString( i18n( "Type" ) ), QString( i18n( "Id" ) ), QString( i18n( "Title" ) ), QString( i18n( "Author/Editor/Publisher" ) ), QString( i18n( "Year" ) )
    };
static const int ColumnsCount = sizeof( Columns ) / sizeof( Columns[ 0 ] );

class KBibTeXListView : public KListView
{
    Q_OBJECT
public:
    KBibTeXListView( KBibTeXPart *kbibtexPart, QWidget *parent = 0, const char *name = 0 );
    ~KBibTeXListView();

    bool open( const KURL &url, bool insertOnly = FALSE );
    bool open( QIODevice *iodevice, bool insertOnly = FALSE, const QString &label = QString::null, BibTeX::BibTeXFileImporter *importer = NULL );
    bool save( const KURL &url, Settings::FileFormat format = Settings::formatBibTeX, bool onlySelected = FALSE );
    bool save( QIODevice *iodevice, Settings::FileFormat format = Settings::formatBibTeX, const QString &label = QString::null, bool onlySelected = FALSE );

    void deleteSelected();
    void copySelected();
    void cutSelected();
    void paste();

    bool newElement( const QString& elementType );
    bool editElement( KBibTeXListViewItem * item, bool isReadWrite = TRUE );

    //     BibTeX::BibTeXFile *selectedElements();

    void enabledActions();
    bool KBibTeXListView::acceptDrag( QDropEvent *event ) const;

public slots:
    void filterText( const QString&text );
    void updateView();

private slots:
    void setSortingColumn( int column );
    void columnSizeChange();
    void showProgress( int current, int total );

protected:
    //     void resizeEvent( QResizeEvent * e );
    void dragEnterEvent( QDragEnterEvent *e );
    void contentsDragEnterEvent( QDragEnterEvent *e );
    void dropEvent( QDropEvent *e );
    void contentsDropEvent( QDropEvent *e );
    void startDrag();

private:
    KBibTeXPart *m_kbibtexPart;
    BibTeX::BibTeXFile *m_bibtexfile;
    QString m_filter;
    QProgressDialog *m_progressDialog;
    int m_newElementCounter;

    void clearAll();
    void setItems();
    void updateVisiblity();
    void appendText( const QString &text );
    void appendBibTeXFile( BibTeX::BibTeXFile *bibtexfile );
    void startProgress( const QString &label, QObject *progressFrom );
    void endProgress( QObject *progressFrom );
    QString selectedAsString();
    BibTeX::BibTeXFile *selected();
};

#endif
