/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qhbuttongroup.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <klocale.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kdialog.h>

#include "settings.h"
#include "kbibtexsettingsfileio.h"

static const QString LanguagesI18N[] =
    {
        i18n( "English" ), i18n( "German" ), i18n( "French" ), i18n( "Swedish" )
    };

static const QString Languages[] =
    {
        QString( "english" ), QString( "german" ), QString( "french" ), QString( "swedish" )
    };
static const int LanguageCount = sizeof( Languages ) / sizeof( Languages[ 0 ] );

static const QString BibliographyStyles[] =
    {
        QString( "plain" ), QString( "siam" ), QString( "alpha" ), QString( "apalike" ), QString( "abbrv" ), QString( "unsrt" ), QString( "ieeetr" )
    };
static const int BibliographyStylesCount = sizeof( BibliographyStyles ) / sizeof( BibliographyStyles[ 0 ] );


KBibTeXSettingsFileIO::KBibTeXSettingsFileIO( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    setupGUI();
}

KBibTeXSettingsFileIO::~KBibTeXSettingsFileIO()
{
    // nothing
}

void KBibTeXSettingsFileIO::applyData()
{
    if ( m_radioButtonEncodingLaTeX->isChecked() )
        kbibtexsettings.encoding = Settings::encLaTeX;
    else if ( m_radioButtonEncodingUTF8->isChecked() )
        kbibtexsettings.encoding = Settings::encUTF8;
    kbibtexsettings.exportlanguage = Languages[ m_comboBoxLanguage->currentItem() ];
    kbibtexsettings.exportbibliographystyle = m_comboBoxBibliographyStyle->currentText();

    if ( m_comboBoxExportSystemHTML->isEnabled() )
    {
        QString text = m_comboBoxExportSystemHTML->currentText();
        /*if ( text.compare( "bib2xhtml" ) == 0 )
            kbibtexsettings.usedExporterHTML = Settings::exporterBib2XHTML;
        else*/ if    (    text.compare(    "bibtex2html" )    ==    0 ) 
            kbibtexsettings.usedExporterHTML = Settings::exporterBibTeX2HTML;
        else if ( text.compare( "bibconv" ) == 0 )
            kbibtexsettings.usedExporterHTML = Settings::exporterBibConv;
        else
            kbibtexsettings.usedExporterHTML = Settings::exporterXSLT;
    }
    else
        kbibtexsettings.usedExporterHTML = Settings::exporterNone;

    if ( m_comboBoxExportSystemEndNote->isEnabled() && m_comboBoxExportSystemEndNote->currentText().compare( "bibconv" ) == 0 )
        kbibtexsettings.usedExporterEndNote = Settings::exporterBibConv;
    else
        kbibtexsettings.usedExporterEndNote = Settings::exporterNone;
    if ( m_comboBoxExportSystemProCite->isEnabled() && m_comboBoxExportSystemProCite->currentText().compare( "bibconv" ) == 0 )
        kbibtexsettings.usedExporterProCite = Settings::exporterBibConv;
    else
        kbibtexsettings.usedExporterProCite = Settings::exporterNone;
}

void KBibTeXSettingsFileIO::setData()
{
    m_radioButtonEncodingLaTeX->setChecked( kbibtexsettings.encoding == Settings::encLaTeX );
    m_radioButtonEncodingUTF8->setChecked( kbibtexsettings.encoding == Settings::encUTF8 );

    for ( int i = 0; i < LanguageCount; i++ )
        if ( Languages[ i ].compare( kbibtexsettings.exportlanguage ) == 0 )
        {
            m_comboBoxLanguage->setCurrentItem( i );
            break;
        }

    m_comboBoxBibliographyStyle->setCurrentText( kbibtexsettings.exportbibliographystyle );
    //     m_comboBoxExternalProgram->setCurrentText( kbibtexsettings.externalHTMLprogram );

    m_comboBoxExportSystemHTML->clear();
    m_comboBoxExportSystemEndNote->clear();
    m_comboBoxExportSystemProCite->clear();

    m_comboBoxExportSystemHTML->insertItem( i18n( "XSLT Stylesheet" ) );
    if ( kbibtexsettings.usedExporterHTML == Settings::exporterXSLT )
        m_comboBoxExportSystemHTML->setCurrentItem( m_comboBoxExportSystemHTML->count() - 1 );
    /*    if ( kbibtexsettings.bib2xhtmlAvailable )
        {
            m_comboBoxExportSystemHTML->insertItem( "bib2xhtml" );
            if ( kbibtexsettings.usedExporterHTML == Settings::exporterBib2XHTML )
                m_comboBoxExportSystemHTML->setCurrentItem( m_comboBoxExportSystemHTML->count() - 1 );
        }*/
    if ( kbibtexsettings.bibtex2htmlAvailable )
    {
        m_comboBoxExportSystemHTML->insertItem( "bibtex2html" );
        if ( kbibtexsettings.usedExporterHTML == Settings::exporterBibTeX2HTML )
            m_comboBoxExportSystemHTML->setCurrentItem( m_comboBoxExportSystemHTML->count() - 1 );
    }
    if ( kbibtexsettings.bibconvAvailable )
    {
        m_comboBoxExportSystemHTML->insertItem( "bibconv" );
        if ( kbibtexsettings.usedExporterHTML == Settings::exporterBibConv )
            m_comboBoxExportSystemHTML->setCurrentItem( m_comboBoxExportSystemHTML->count() - 1 );
        m_comboBoxExportSystemEndNote->insertItem( "bibconv" );
        if ( kbibtexsettings.usedExporterEndNote == Settings::exporterBibConv )
            m_comboBoxExportSystemEndNote->setCurrentItem( m_comboBoxExportSystemEndNote->count() - 1 );
        m_comboBoxExportSystemProCite->insertItem( "bibconv" );
        if ( kbibtexsettings.usedExporterProCite == Settings::exporterBibConv )
            m_comboBoxExportSystemProCite->setCurrentItem( m_comboBoxExportSystemProCite->count() - 1 );
    }

    if ( m_comboBoxExportSystemHTML->count() == 0 )
    {
        m_comboBoxExportSystemHTML->insertItem( i18n( "No exporter available" ) );
        m_comboBoxExportSystemHTML->setEnabled( FALSE );
    }
    if ( m_comboBoxExportSystemEndNote->count() == 0 )
    {
        m_comboBoxExportSystemEndNote->insertItem( i18n( "No exporter available" ) );
        m_comboBoxExportSystemEndNote->setEnabled( FALSE );
    }
    if ( m_comboBoxExportSystemProCite->count() == 0 )
    {
        m_comboBoxExportSystemProCite->insertItem( i18n( "No exporter available" ) );
        m_comboBoxExportSystemProCite->setEnabled( FALSE );
    }
}

void KBibTeXSettingsFileIO::slotConfigChanged()
{
    emit configChanged();
}

void KBibTeXSettingsFileIO::setupGUI()
{
    QVBoxLayout * layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );

    m_buttonGroupEncoding = new QHButtonGroup( i18n( "Encoding of non-ASCII characters" ), this );
    layout->addWidget( m_buttonGroupEncoding );
    QWhatsThis::add( m_buttonGroupEncoding, i18n( "<qt><p>Select the encoding when saving BibTeX files.</p>\n<p>Select <strong>LaTeX style</strong> for better compatibility or <strong>UTF-8</strong> for better readabilty on modern systems.</p></qt>" ) );
    m_radioButtonEncodingLaTeX = new QRadioButton( i18n( "&LaTeX style" ), m_buttonGroupEncoding );
    m_buttonGroupEncoding->insert( m_radioButtonEncodingLaTeX );
    m_radioButtonEncodingUTF8 = new QRadioButton( i18n( "&UTF-8" ), m_buttonGroupEncoding );
    m_buttonGroupEncoding->insert( m_radioButtonEncodingUTF8 );

    QGroupBox *groupBox = new QGroupBox( 2, Horizontal, i18n( "PDF and PostScript Export" ), this );
    layout->addWidget( groupBox );
    QWhatsThis::add( groupBox, i18n( "<qt><p>Select the layout of the resulting PDF or PostScript document when exporting a BibTeX file.</p></qt>" ) );
    QLabel *label = new QLabel( i18n( "&Language:" ), groupBox );
    m_comboBoxLanguage = new QComboBox( groupBox );
    label->setBuddy( m_comboBoxLanguage );
    label = new QLabel( i18n( "&Biblography style:" ), groupBox );
    m_comboBoxBibliographyStyle = new QComboBox( TRUE, groupBox );
    label->setBuddy( m_comboBoxBibliographyStyle );

    groupBox = new QGroupBox( 2, Horizontal, i18n( "Export Systems" ), this );
    layout->addWidget( groupBox );
    label = new QLabel( i18n( "HTML:" ), groupBox );
    m_comboBoxExportSystemHTML = new QComboBox( FALSE, groupBox );
    label->setBuddy( m_comboBoxExportSystemHTML );
    label = new QLabel( i18n( "EndNote:" ), groupBox );
    m_comboBoxExportSystemEndNote = new QComboBox( FALSE, groupBox );
    label->setBuddy( m_comboBoxExportSystemEndNote );
    label = new QLabel( i18n( "ProCite:" ), groupBox );
    m_comboBoxExportSystemProCite = new QComboBox( FALSE, groupBox );
    label->setBuddy( m_comboBoxExportSystemProCite );

    layout->addStretch( 1 );

    for ( int i = 0; i < LanguageCount; i++ )
        m_comboBoxLanguage->insertItem( LanguagesI18N[ i ] );
    for ( int i = 0; i < BibliographyStylesCount; i++ )
        m_comboBoxBibliographyStyle->insertItem( BibliographyStyles[ i ] );

    connect( m_buttonGroupEncoding, SIGNAL( clicked( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( m_comboBoxLanguage, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( m_comboBoxBibliographyStyle, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( m_comboBoxExportSystemHTML, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( m_comboBoxExportSystemEndNote, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( m_comboBoxExportSystemProCite, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
}

#include "kbibtexsettingsfileio.moc"
