/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qbuffer.h>
#include <kdialogbase.h>
#include <klocale.h>

#include <bibtexstring.h>
#include <bibtexelement.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileimporterbibtex.h>

#include "kbibtexstringwidget.h"

KBibTeXStringWidget::KBibTeXStringWidget( BibTeX::BibTeXString *bibtexstring, bool isReadWrite, QWidget *parent, const char *name )
        : QTabWidget( parent, name ), m_bibtexstring( bibtexstring )
{
    setupGUI();
    getStringData();
    m_lineEditStringId->setReadOnly( !isReadWrite );
    m_multiLineEditStringValue->setReadOnly( !isReadWrite );
}

KBibTeXStringWidget::~KBibTeXStringWidget()
{
    // nothing
}

void KBibTeXStringWidget::setupGUI()
{
    setMinimumWidth( 384 );

    QWidget *normalEdit = new QWidget( this, "normalEdit" );
    insertTab( normalEdit, i18n( "Normal" ) );
    QVBoxLayout * layout = new QVBoxLayout( normalEdit, KDialog::marginHint(), KDialog::spacingHint() );

    QLabel *label = new QLabel( i18n( "String &id:" ), normalEdit );
    layout->addWidget( label );
    m_lineEditStringId = new QLineEdit( normalEdit );
    layout->addWidget( m_lineEditStringId );
    label->setBuddy( m_lineEditStringId );

    label = new QLabel( i18n( "String &value:" ), normalEdit );
    layout->addWidget( label );
    m_multiLineEditStringValue = new QMultiLineEdit( normalEdit );
    layout->addWidget( m_multiLineEditStringValue );
    label->setBuddy( m_multiLineEditStringValue );

    m_sourceTab = new QWidget( this, "sourceTab" );
    insertTab( m_sourceTab, i18n( "Source" ) );
    layout = new QVBoxLayout( m_sourceTab, KDialog::marginHint(), KDialog::spacingHint() );

    m_textEditSource = new QTextEdit( m_sourceTab, "sourceEdit" );
    layout->addWidget( m_textEditSource );

    connect( this, SIGNAL( currentChanged( QWidget* ) ), this, SLOT( slotCurrentTabChanged( QWidget* ) ) );

}

void KBibTeXStringWidget::setStringData()
{
    QString text = m_lineEditStringId->text();
    m_bibtexstring->setKey( text );

    m_bibtexstring->clearStringItems();
    BibTeX::BibTeXElement::FieldItem item;
    item.text = m_multiLineEditStringValue->text();
    item.isStringKey = FALSE;
    m_bibtexstring->appendStringItem( item );
}

void KBibTeXStringWidget::getStringData()
{
    m_lineEditStringId->setText( m_bibtexstring->key() );

    if ( m_bibtexstring->begin() != m_bibtexstring->end() )
    {
        BibTeX::BibTeXElement::FieldItem item = *( m_bibtexstring->begin() );
        m_multiLineEditStringValue->setText( item.text );
    }
}

void KBibTeXStringWidget::slotCurrentTabChanged( QWidget* newTab )
{
    if ( newTab == m_sourceTab )
    {
        // switching from normal edit to source edit
        BibTeX::BibTeXString * string = new BibTeX::BibTeXString( m_lineEditStringId->text() );
        BibTeX::BibTeXElement::FieldItem item;
        item.text = m_multiLineEditStringValue->text();
        item.isStringKey = FALSE;
        string->appendStringItem( item );

        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeX::BibTeXFileExporter * exporter = new BibTeX::BibTeXFileExporterBibTeX();
        if ( exporter->save( &buffer, string ) )
        {
            buffer.close();

            buffer.open( IO_ReadOnly );
            m_textEditSource->setText( QTextStream( &buffer ).read() );
        }
        else
            m_textEditSource->setText( "" );
        buffer.close();
        delete string;
        delete exporter;
    }
    else
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        QTextStream( &buffer ) << m_textEditSource->text() << endl;
        buffer.close();

        buffer.open( IO_ReadOnly );
        BibTeX::BibTeXFileImporter *importer = new BibTeX::BibTeXFileImporterBibTeX();
        BibTeX::BibTeXFile *bibtexfile = importer->load( &buffer );
        delete importer;
        buffer.close();

        if ( bibtexfile != NULL )
        {
            BibTeX::BibTeXString * string = bibtexfile->count() == 1 ? dynamic_cast<BibTeX::BibTeXString*>( bibtexfile->at( 0 ) ) : NULL;
            if ( string != NULL )
            {
                m_lineEditStringId->setText( string->key() );
                if ( string->begin() != string->end() )
                {
                    BibTeX::BibTeXElement::FieldItem item = *( string->begin() );
                    m_multiLineEditStringValue->setText( item.text );
                }
            }
            delete bibtexfile;
        }
    }
}

QDialog::DialogCode KBibTeXStringWidget::execute( BibTeX::BibTeXString *bibtexstring, bool isReadWrite, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, i18n( "Edit BibTeX String" ), KDialogBase::Ok | KDialogBase::Cancel );
    KBibTeXStringWidget* ui = new KBibTeXStringWidget( bibtexstring, isReadWrite, dlg, "kbibtexstringwidget" );

    dlg->setMainWidget( ui );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();
    if ( isReadWrite && result == QDialog::Accepted )
        ui->setStringData();

    delete( ui );
    delete( dlg );

    return result;
}

#include "kbibtexstringwidget.moc"
