/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlineedit.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qlabel.h>

#include <kcombobox.h>
#include <klocale.h>
#include <kdialog.h>
#include <kiconloader.h>

#include <settings.h>
#include "searchbar.h"

namespace KBibTeX
{

    SearchBar::SearchBar( QWidget *parent, const char *name )
            : QWidget( parent, name )
    {
        m_timerInput = new QTimer( this );
        connect( m_timerInput, SIGNAL( timeout() ), SLOT( slotTimeout() ) );

        setupGUI();
    }

    SearchBar::~SearchBar()
    {
        delete m_timerInput;
    }

    void SearchBar::setAddElementMenu( QPopupMenu *menu )
    {
        m_pushButtonAddElement->setPopup( menu );
        m_comboboxFilter->setHistoryItems( kbibtexsettings.filterHistory );
    }

    void SearchBar::setupGUI()
    {
        QBoxLayout * layout = new QHBoxLayout( this, 2 /* KDialog::marginHint()*/, KDialog::spacingHint() );

        m_pushButtonAddElement = new QPushButton( this );
        m_pushButtonAddElement->setIconSet( QIconSet( SmallIcon( "filenew" ) ) );
        layout->addWidget( m_pushButtonAddElement );
        QPopupMenu *menu = new QPopupMenu( this );
        menu->insertSeparator();
        m_pushButtonAddElement->setPopup( menu );

        m_pushButtonClearSearchText = new QPushButton( this );
        m_pushButtonClearSearchText->setIconSet( QIconSet( SmallIcon( "locationbar_erase" ) ) );
        layout->addWidget( m_pushButtonClearSearchText );

        QLabel *label = new QLabel( i18n( "Search:" ), this );
        layout->addWidget( label );

        m_comboboxFilter = new KHistoryCombo( TRUE, this, "search_combobox" );
        layout->addWidget( m_comboboxFilter );
        label->setBuddy( m_comboboxFilter );
        m_comboboxFilter->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Preferred );
        m_comboboxFilter->setMaxCount( 256 );

        connect( m_comboboxFilter, SIGNAL( activated( const QString& ) ), m_comboboxFilter, SLOT( addToHistory( const QString& ) ) );

        connect( m_pushButtonClearSearchText, SIGNAL( clicked() ), this, SLOT( slotClear() ) );
        connect( m_comboboxFilter, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotAnnounceDoSearch() ) );
        connect( m_comboboxFilter->lineEdit(), SIGNAL( returnPressed() ), this, SLOT( slotAnnounceDoSearch() ) );
    }

    void SearchBar::slotClear()
    {
        m_comboboxFilter->lineEdit() ->clear();
    }

    void SearchBar::slotTimeout()
    {
        kbibtexsettings.filterHistory = m_comboboxFilter->historyItems();
        emit doSearch( m_comboboxFilter->currentText() );
    }

    void SearchBar::slotAnnounceDoSearch()
    {
        if ( m_timerInput->isActive() )
            m_timerInput->stop();

        m_timerInput->start( 0, true );
    }

}
#include "searchbar.moc"
