/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef SETTINGS_H
#define SETTINGS_H

class KConfig;
class KStandardDirs;
class QString;
class QStringList;

class Settings
{
public:
    enum Encoding { encLaTeX = 1, encUTF8 = 2 };
    enum Exporter { exporterNone = 0, exporterXSLT = 1, exporterBib2XHTML = 2, exporterBibTeX2HTML = 3, exporterBibConv = 4 };
    enum Importer {importerNone = 0, importerBibConv = 4};
    enum FileFormat { formatUndefined, formatBibTeX, formatXML, formatHTML, formatPDF, formatPS, formatEndNote, formatProCite };

    struct SearchURL
    {
        QString description;
        QString url;
    };

    Settings();
    ~Settings();

    void load( KConfig * config );
    void save( KConfig * config );

    Encoding encoding;
    QString exportlanguage;
    QString exportbibliographystyle;
    bool enableallfields;
    int mainListSortingColumn;
    int mainListColumnWidth[ 5 ];
    QPtrList<SearchURL> searchURLs;

    QString xsltStylesheetHTML;

    bool bibconvAvailable;
    bool bibtex2htmlAvailable;
    bool bib2xhtmlAvailable;

    Exporter usedExporterHTML;
    Exporter usedExporterEndNote;
    Exporter usedExporterProCite;

    QStringList filterHistory;

    bool showStrings;
    bool showComments;

    void setDefaultSearchURLs();

private:
    void checkExternalToolsAvailable();
    bool checkExternalToolAvailable( const QString &binary );

    KStandardDirs *m_standardDirs;

};

extern Settings kbibtexsettings;

#endif
